﻿using System;
using System.Collections.Generic;

#if NETFX_CORE

using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

#endif

#if WINDOWS_PHONE

using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

#endif

namespace BayanControlsComponent
{
    public sealed class Key : ContentControl
    {
        static readonly DependencyProperty isPressedProperty =
                    DependencyProperty.Register("IsPressed",
                        typeof(bool), typeof(Key),
                        new PropertyMetadata(false, OnIsPressedChanged));

        public event EventHandler<bool> IsPressedChanged;

        List<uint> pointerList = new List<uint>();

        public Key()
        {
            this.DefaultStyleKey = typeof(Key);

#if WINDOWS_PHONE

            this.Loaded += (sender, args) =>
                {
                    Touch.FrameReported += OnTouchFrameReported;
                };

            this.Unloaded += (sender, args) =>
                {
                    Touch.FrameReported -= OnTouchFrameReported;
                };

#endif
        }

        public static DependencyProperty IsPressedProperty
        {
            get { return isPressedProperty; }
        }

        public bool IsPressed
        {
            set { SetValue(IsPressedProperty, value); }
            get { return (bool)GetValue(IsPressedProperty); }
        }

#if WINDOWS_PHONE

        void OnTouchFrameReported(object sender, TouchFrameEventArgs args)
        {
            TouchPointCollection touchPoints = args.GetTouchPoints(this);

            foreach (TouchPoint touchPoint in touchPoints)
            {
                uint id = (uint)touchPoint.TouchDevice.Id;
                bool overThis = IsOverThis(touchPoint.TouchDevice.DirectlyOver);

                switch (touchPoint.Action)
                {
                    case TouchAction.Down:
                        if (overThis)
                            AddToList(id);

                        break;

                    case TouchAction.Move:
                        if (overThis)
                            AddToList(id);
                        else
                            RemoveFromList(id);

                        break;

                    case TouchAction.Up:
                        if (overThis)
                            RemoveFromList(id);

                        break;
                }
            }
        }

        bool IsOverThis(UIElement element)
        {
            if (element == this)
                return true;

            element = VisualTreeHelper.GetParent(element) as UIElement;

            if (element == null)
                return false;

            return IsOverThis(element);
        }

#endif
        
#if NETFX_CORE

        protected override void OnPointerEntered(PointerRoutedEventArgs args)
        {
            if (args.Pointer.IsInContact)
                AddToList(args.Pointer.PointerId);

            base.OnPointerEntered(args);
        }

        protected override void OnPointerPressed(PointerRoutedEventArgs args)
        {
            AddToList(args.Pointer.PointerId);
            base.OnPointerPressed(args);
        }

        protected override void OnPointerReleased(PointerRoutedEventArgs args)
        {
            RemoveFromList(args.Pointer.PointerId);
            base.OnPointerReleased(args);
        }

        protected override void OnPointerExited(PointerRoutedEventArgs args)
        {
            RemoveFromList(args.Pointer.PointerId);
            base.OnPointerExited(args);
        }

#endif

        void AddToList(uint id)
        {
            if (!pointerList.Contains(id))
                pointerList.Add(id);

            CheckList();
        }

        void RemoveFromList(uint id)
        {
            if (pointerList.Contains(id))
                pointerList.Remove(id);

            CheckList();
        }

        void CheckList()
        {
            this.IsPressed = pointerList.Count > 0;
        }

        static void OnIsPressedChanged(DependencyObject obj,
                                       DependencyPropertyChangedEventArgs args)
        {
            Key key = (Key)obj;

            if (key.IsPressedChanged != null)
                key.IsPressedChanged(key, key.IsPressed);

            VisualStateManager.GoToState(key, key.IsPressed ? "Pressed" : "Normal", false);
        }
    }
}
