﻿using System;

#if NETFX_CORE

using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

#endif

#if WINDOWS_PHONE

using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

#endif

namespace BayanControlsComponent
{
    public sealed partial class Keyboard : UserControl
    {
        // 14, 18, 22 are good values
        static readonly DependencyProperty columnCountProperty =
                    DependencyProperty.Register("ColumnCount",
                        typeof(int), typeof(Keyboard),
                        new PropertyMetadata(18, OnPropertyChanged));

        // Set to 1 if using fewer buttons
        static readonly DependencyProperty octaveTransposeProperty =
                    DependencyProperty.Register("OctaveTranspose",
                        typeof(int), typeof(Keyboard),
                        new PropertyMetadata(0, OnPropertyChanged));

        public event EventHandler<KeyArgs> KeyPressedChanged;

        public Keyboard()
        {
            InitializeComponent();
            CreateKeys();
        }

        public static DependencyProperty ColumnCountProperty
        {
            get { return columnCountProperty; }
        }

        public static DependencyProperty OctaveTransposeProperty
        {
            get { return octaveTransposeProperty; }
        }

        public int ColumnCount
        {
            set { SetValue(ColumnCountProperty, value); }
            get { return (int)GetValue(ColumnCountProperty); }
        }

        public int OctaveTranspose
        {
            set { SetValue(OctaveTransposeProperty, value); }
            get { return (int)GetValue(OctaveTransposeProperty); }
        }

        static void OnPropertyChanged(DependencyObject obj,
                                      DependencyPropertyChangedEventArgs args)
        {
            ((Keyboard)obj).CreateKeys();
        }

        void CreateKeys()
        {
            layoutCanvas.Children.Clear();

            int[,] keyNums = new int[5, 4] { { 37, 40, 43, 46 }, 
                                             { 35, 38, 41, 44 },
                                             { 36, 39, 42, 45 },
                                             { 34, 37, 40, 43 },
                                             { 35, 38, 41, 44 } };

            string[] notes = { "C", "C#", "D", "D#", "E", "F", 
                               "F#", "G", "G#", "A", "A#", "B" };

            Brush whiteBrush = new SolidColorBrush(Colors.White);
            Brush blackBrush = new SolidColorBrush(Colors.Black);
            Brush grayBrush = new SolidColorBrush(Color.FromArgb(255, 192, 192, 192));

            layoutCanvas.Width = 100 * this.ColumnCount;
            layoutCanvas.Height = 500;

            for (int row = 0; row < 5; row++)
            {
                bool isEvenRow = (row & 1) == 0;
                int cols = this.ColumnCount - (isEvenRow ? 1 : 0);

                for (int col = 0; col < cols; col++)
                {
                    int keyNum = keyNums[row, col % 4] + 12 * (col / 4 + OctaveTranspose);
                    string note = notes[keyNum % 12];

                    Key key = new Key();
                    key.Tag = row * 1000 + keyNum;
                    key.Content = note;
                    key.Foreground = blackBrush;
                    key.Background = note.Length == 2 ? grayBrush : whiteBrush;
                    key.Width = 90;
                    key.Height = 90;
                    key.FontSize = 48;
                    key.IsPressedChanged += OnKeyIsPressedChanged;

                    Canvas.SetTop(key, row * 100 + 5);
                    Canvas.SetLeft(key, col * 100 + (isEvenRow ? 55 : 5));
                    layoutCanvas.Children.Add(key);
                }
            }
        }

        void OnKeyIsPressedChanged(object sender, bool isPressed)
        {
            KeyArgs keyArgs = new KeyArgs
            {
                KeyNumber = (int)((Key)sender).Tag,
                IsPressed = isPressed
            };

            if (KeyPressedChanged != null)
                KeyPressedChanged(this, keyArgs);
        }
    }
}
